/*
 * =====================================================================================
 * 
 *     Modulename:
 *       Filename:  app_english_watch.c
 *
 *    Description:  英文表盘 
 *    Corporation:
 * 
 *         Author:  gliu (), gliu@damaijiankang.com
 *        Created:  2015年03月30日 19时48分47秒
 *
 * =====================================================================================
 *
 * =====================================================================================
 * 
 *   MODIFICATION HISTORY :
 *    
 *		     DATE :
 *		     DESC :
 * =====================================================================================
 */	
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#ifdef LINUX
#include <unistd.h>
#include <time.h>
#include <pthread.h>
#include "linux_screen_show.h"
#include "os_store_manage.h"
#include "os_time.h"
#include "os_comm.h"
#include "os_res_bitmap.h"
#include "os_sys_app_id.h"
#include "os_memory_manage.h"
#include "res_user_bitmap.h"
#include "os_sys_event.h"
#include "window_stack.h"
#include "window.h"
#include "screen_show.h"
#include "matrix.h"
#include "plug_status_bar.h"
#include "res_bitmap_base.h"

#else
#include "maibu_sdk.h"
#include "maibu_res.h"

#endif



/*窗口ID, 通过该窗口ID获取窗口句柄*/
static int32_t g_app_english_window_id = -1;

/*图层ID*/
static int8_t g_app_english_block_layer_id = -1;



/*定义各个图层的位置*/

/*背景图层*/
#define ENGLISH_BG_ORIGIN_X		0
#define ENGLISH_BG_ORIGIN_Y		0
#define ENGLISH_BG_SIZE_H		128
#define ENGLISH_BG_SIZE_W		128

/*显示小时文本图层*/
#define ENGLISH_H_ORIGIN_X		8
#define ENGLISH_H_ORIGIN_Y		20
#define ENGLISH_H_SIZE_H		30		
#define ENGLISH_H_SIZE_W		120

/*显示分钟十位文本图层*/
#define ENGLISH_M1_ORIGIN_X		8
#define ENGLISH_M1_ORIGIN_Y		51
#define ENGLISH_M1_SIZE_H		24
#define ENGLISH_M1_SIZE_W		120

/*显示分钟个位文本图层*/
#define ENGLISH_M2_ORIGIN_X		8
#define ENGLISH_M2_ORIGIN_Y		76
#define ENGLISH_M2_SIZE_H		20
#define ENGLISH_M2_SIZE_W		120




const char english_hour_array[][10] = 
{
	"twelve",
	"one",
	"two",
	"three",
	"four",				
	"five",
	"six",
	"seven",
	"eight",
	"nine",
	"ten",
	"eleven",
	"twelve",
	"thirteen",	
	"fourteen",
	"fifteen",
	"sixteen",
	"seventeen",
	"eighteen",
	"nineteen",
	"twenty"
};

const char english_min_array[][8] = 
{
	"twenty",
	"thirty",
	"forty",
	"fifty"
};




void app_english_watch_update(int8_t hour, int8_t min, int8_t flag)
{

	/*根据窗口ID获取窗口句柄*/
	P_Window p_window = app_window_stack_get_window_by_id(g_app_english_window_id);	
	if (NULL == p_window)
	{
		return;
	}


	P_Layer p_layer = app_window_get_layer_by_id(p_window, g_app_english_block_layer_id);
	if (NULL != p_layer)
	{
		/*如果分钟为0，则小时显示x o'clock*/
		char str[20] = "";
		if (min == 0)
		{
			strcpy(str, "o'clock");
		}
		else if (min < 21)
		{
			strcpy(str, english_hour_array[min]);
		}
		else
		{
			strcpy(str, english_min_array[min/10 - 2]);
		}
		app_layer_set_block_element_text(p_layer, 2, str, sizeof(str));


		if ((min > 20)&&((min%10) != 0))
		{
			memset(str, 0, sizeof(str));
			strcpy(str, english_hour_array[min%10]);
			app_layer_set_block_element_text(p_layer, 3, str, sizeof(str));
		}
		else
		{
			memset(str, 0, sizeof(str));
			app_layer_set_block_element_text(p_layer, 3, str, sizeof(str));
		}


		if ((flag == 1) || (min == 0))	
		{
			memset(str, 0, sizeof(str));
			sprintf(str, "%s", english_hour_array[hour%12]);
			app_layer_set_block_element_text(p_layer, 1, str, sizeof(str));

		}

		app_window_update(p_window);
	}
	

}



void app_english_watch_time_change(enum SysEventType type, void *context)
{
	/*时间更改*/
	if (type == SysEventTypeTimeChange)
	{
		struct date_time t;
		app_service_get_datetime(&t);
		app_english_watch_update(t.hour, t.min, 1);
	}
}


#if 0
/*
 *--------------------------------------------------------------------------------------
 *     function:  app_english_timer_callback
 *    parameter: 
 *       return:
 *  description:  定时更新时间
 * 	  other:
 *--------------------------------------------------------------------------------------
 */
void app_english_timer_callback(date_time_t tick_time, uint32_t millis, void *context)
{
	app_english_watch_update(tick_time->hour, tick_time->min, 0);
}
#endif


P_Window init_english_window()
{
	P_Window p_window = NULL;
	p_window = app_window_create();
	if (NULL == p_window)
	{
		return NULL;
	}


	/*添加块*/
	GRect frame_bg = {{ENGLISH_BG_ORIGIN_X, ENGLISH_BG_ORIGIN_Y}, {ENGLISH_BG_SIZE_H, ENGLISH_BG_SIZE_W}};
	LayerBlock struct_block= {frame_bg};
	BElement element;		

	struct date_time t;
	app_service_get_datetime(&t);
	GBitmap bitmap_bg;

	/*背景图片*/
	memset(&element, 0, sizeof(BElement));
	element.type = BELEMENT_TYPE_BITMAP;
	element.color = GColorBlack;
	element.frame.origin.x = ENGLISH_BG_ORIGIN_X;
	element.frame.origin.y = ENGLISH_BG_ORIGIN_Y;
	element.frame.size.h = ENGLISH_BG_SIZE_H;	
	element.frame.size.w = ENGLISH_BG_SIZE_W;	
	res_get_user_bitmap(RES_BITMAP_WATCHFACE_ENGLISH_BG, &bitmap_bg);
	element.context.bmp = bitmap_bg;
	app_layer_add_block_element(&struct_block, &element);

	/*小时*/
	memset(&element, 0, sizeof(BElement));
	element.type = BELEMENT_TYPE_TEXT;
	element.color = GColorWhite;
	element.frame.origin.x = ENGLISH_H_ORIGIN_X;
	element.frame.origin.y = ENGLISH_H_ORIGIN_Y;
	element.frame.size.h = ENGLISH_H_SIZE_H;	
	element.frame.size.w = ENGLISH_H_SIZE_W;	
	element.font = U_ASCII_ARIALBD_30;
	element.alignment = GAlignLeft;
	sprintf(element.context.text, "%s", english_hour_array[t.hour%12]);
	app_layer_add_block_element(&struct_block, &element);


	/*分钟第一个数字*/
	memset(&element, 0, sizeof(BElement));
	element.type = BELEMENT_TYPE_TEXT;
	element.color = GColorWhite;
	element.frame.origin.x = ENGLISH_M1_ORIGIN_X;
	element.frame.origin.y = ENGLISH_M1_ORIGIN_Y;
	element.frame.size.h = ENGLISH_M1_SIZE_H;	
	element.frame.size.w = ENGLISH_M1_SIZE_W;	
	element.font = U_ASCII_ARIALBD_24;
	element.alignment = GAlignLeft;
	/*如果分钟为0，则小时显示x o'clock*/
	if (t.min == 0)
	{
		strcpy(element.context.text, "o'clock");
	}
	else if (t.min < 21)
	{
		strcpy(element.context.text, english_hour_array[t.min]);
	}
	else
	{
		strcpy(element.context.text, english_min_array[t.min/10 - 2]);
	}
	app_layer_add_block_element(&struct_block, &element);


	/*添加分钟个位图层*/
	if ((t.min > 20)&&((t.min%10) != 0))
	{
		memset(&element, 0, sizeof(BElement));
		element.type = BELEMENT_TYPE_TEXT;
		element.color = GColorWhite;
		element.frame.origin.x = ENGLISH_M2_ORIGIN_X;
		element.frame.origin.y = ENGLISH_M2_ORIGIN_Y;
		element.frame.size.h = ENGLISH_M2_SIZE_H;	
		element.frame.size.w = ENGLISH_M2_SIZE_W;	
		element.font = U_ASCII_ARIALBD_20;
		element.alignment = GAlignLeft;
		strcpy(element.context.text, english_hour_array[t.min%10]);
		app_layer_add_block_element(&struct_block, &element);
	}


	/*创建块图层*/
	P_Layer layer = app_layer_create_block(&struct_block);
	if(layer != NULL)
	{
		g_app_english_block_layer_id = app_window_add_layer(p_window, layer);
	}

	/*添加窗口定时器，定时更新*/
	//app_window_timer_subscribe(p_window, 60000, app_english_timer_callback, NULL);

	/*注册一个事件通知回调，当有时间改变是，立即更新时间*/
	maibu_service_sys_event_subscribe(app_english_watch_time_change);

	return p_window;

}




int main()
{

#ifdef LINUX	
	/*非APP编写*/	
	screen_init(SCREEN_ROW_NUMS,SCREEN_COL_NUMS);
	os_store_manage_init();	
	window_stack_init();
	SHOW;
#endif


	/*创建显示英文窗口*/
	P_Window p_window = init_english_window(); 
	if (p_window != NULL)
	{
		/*放入窗口栈显示k*/
		g_app_english_window_id = app_window_stack_push(p_window);
	}


#ifdef LINUX
	SHOW;
	/*非APP编写*/	
	while (1)
	{
		char input;	
	
		/*输入操作*/
		scanf("%c", &input);
		if (input == 'q')
		{
			break;	
		}
		else if (input == 'c')
		{
	
		}
		else if (input == 'g')
		{
			break;
		}
	
	}	

	app_window_stack_pop(p_window);
	window_stack_destory();
	screen_destory();
	os_store_manage_destory();

	SHOW;

#endif

	return 0;

}



